/**
* @overview Scalex/Guitar Toolkit Web App
* @author Christopher Cook
* @copyright Webprofusion Ltd http://webprofusion.com
*/
/// <reference path="../Scripts/typings/jquery/jquery.d.ts" />
/// <reference path="../Scripts/typings/phonegap/phonegap.d.ts" />

var currentAppSettings = null;
var scaleRenderer = null;
var scaleCanvas = null;
var chordCanvas = null;
var chordRenderer = null;
var initialised_scales = false;
var initialised_chords = false;
var audioController = null;
var scalePortraitMode = false;
var enableTuning = false;
var audioReady = false;
var playMetronome = false;
var debugMode = false;
var isCanvasMode = true;
var guitarModel = null;
var backButtonCount = 0;
var timeLastBackButton = null;
var isCordova = false;

function logEvent(msg) {
    if (debugMode) {
        if (window.console)
            console.log(msg);
    }
}

function refreshScaleDiagram() {
    //if ($(window).height() > $(window).width()) scalePortraitMode = true;
    //else scalePortraitMode = true;
    scaleRenderer.GuitarModel.SetScale$$String($("#scale").val());
    scaleRenderer.GuitarModel.SetKey$$String($("#key").val());
    scaleRenderer.GuitarModel.SetTuning$$String($("#tuning").val());
    scaleRenderer.GuitarModel.RefreshStringSettings();

    var w = 440;
    var h = 100;
    var scaleSize = 2;
    var rotateDegrees = 0;

    if (scaleCanvas == null) {
        if (isCanvasMode) {
            $("#scale-diagram-container").removeClass("scaled");
            scaleCanvas = new Webprofusion.Scalex.Rendering.WebCanvas.ctor('scale-diagram-container', w, h, 2);

            scaleCanvas.SetScale(scaleSize, rotateDegrees);
        } else
            scaleCanvas = new Webprofusion.Scalex.Rendering.SVGCanvas.ctor('scale-diagram-container', w, h);
    }

    if (initialised_scales)
        scaleCanvas.Clear();
    scaleCanvas.Initialise();

    scaleRenderer.Render(scaleCanvas, w, h);

    if (isCanvasMode) {
        var fretboardHeight = scaleRenderer.GetFretboardHeight() * scaleCanvas.yscale;
        document.getElementById("scale-diagram-container").style.backgroundSize = "auto " + (fretboardHeight + 10) + "px";
    } else {
        scaleCanvas.SetScale(scaleSize * 2, rotateDegrees);
    }

    initialised_scales = true;

    changeOrientation();
}

function changeOrientation() {
    //landscape mode off/on
    $("#scale-diagram-container").toggleClass("landscape-diagram", !scalePortraitMode);

    //portrait mode off/on
    $("#scale-diagram-container").toggleClass("portrait-diagram", scalePortraitMode);
    $("#carousel_container").toggleClass("carousel_portrait", scalePortraitMode);
}

function refreshChordDiagram() {
    chordRenderer.GuitarModel.SetKey$$String($("#key").val());
    chordRenderer.GuitarModel.SetTuning$$String($("#tuning").val());

    chordRenderer.GuitarModel.RefreshStringSettings();

    chordRenderer.CurrentChordDiagrams = chordRenderer.GuitarModel.GetChordDiagramsByGroup($("#chords").val());

    if (chordCanvas == null) {
        if (isCanvasMode) {
            $("#chord-diagram-container").removeClass("scaled");
            chordCanvas = new Webprofusion.Scalex.Rendering.WebCanvas.ctor('chord-diagram-container', 350, 300, 1.5);
        } else
            chordCanvas = new Webprofusion.Scalex.Rendering.SVGCanvas.ctor('chord-diagram-container', 350, 300);
    }

    if (initialised_chords)
        chordCanvas.Clear();

    chordCanvas.Initialise();

    chordRenderer.Render(chordCanvas, 350, 200);

    //if (!initialised)
    //chordCanvas.SetScale(1);
    initialised_chords = true;

    changeOrientation();
}

function showSideNav() {
    //show side nav
    console.log("showing nav");
    $("#nav-side-menu").fadeIn();
}

function hideSideNav() {
    //show side nav
    console.log("hiding nav");
    $("#nav-side-menu").fadeOut();
}

function hideDialogs() {
    $('#settingsDialog').modal('hide');
}

function initSettings() {
    //setup nav events
    //setup dropdown settings changes
    $(".nav-heading").click(function () {
        showSideNav();
    });
    $("#nav-sidemenu-heading").click(function () {
        hideSideNav();
    });

    //setup dropdown settings changes
    $("#scale").change(function () {
        refreshScaleDiagram();
    });

    $("#key").change(function () {
        refreshScaleDiagram();
        refreshChordDiagram();
    });

    $("#tuning").change(function () {
        refreshScaleDiagram();
        refreshChordDiagram();
    });

    $("#chords").change(function () {
        refreshChordDiagram();
    });

    //setup scale dropdown options
    var scaleDropdown = $('#scale');
    var scaleList = currentAppSettings.get_ScaleManager().GetScaleNameList().GetEnumerator();
    while (scaleList.MoveNext()) {
        var val = scaleList.get_Current();
        scaleDropdown.append($('<option></option>').val(val).html(val));
    }

    //setup tuning dropdown options
    var tuningDropdown = $('#tuning');
    var tuningList = currentAppSettings.get_TuningManager().GetTuningNameList().GetEnumerator();
    while (tuningList.MoveNext()) {
        var val = tuningList.get_Current();
        tuningDropdown.append($('<option></option>').val(val).html(val));
    }

    /////////
    var selectedTuning = currentAppSettings.get_TuningManager().get_AllTunings().get_Item$$Int32(0);
    for (var s = 0; s < selectedTuning.NumberStrings; s++) {
        logEvent(selectedTuning.TuningNotes[s].get_Name() + ":" + selectedTuning.TuningNotes[s].get_NoteFrequency() + ":" + selectedTuning.TuningNotes[s].get_Octave() + "<br/>");
    }

    ///////////
    //setup key dropdown options
    var keyDropdown = $('#key');
    var k = Webprofusion.Scalex.Music.NoteManager.get_NoteNames();
    var keyList = k.GetEnumerator();
    while (keyList.MoveNext()) {
        var val = keyList.get_Current();
        keyDropdown.append($('<option></option>').val(val).html(val));
    }

    //setup chord group dropdown options
    //chordRenderer.GuitarModel = new Webprofusion.Scalex.Guitar.GuitarModel.ctor$$CurrentAppSettings(currentAppSettings);
    //chordRenderer.GuitarModel.SetTuning$$String("6 String Guitar - Standard");
    chordRenderer.CurrentChordDiagrams = chordRenderer.GuitarModel.GetPopularChordDiagrams();

    var chordDropdown = $('#chords');
    var chordGroups = new System.Collections.Generic.List$1.ctor(Webprofusion.Scalex.Music.ChordDefinition.ctor);
    chordGroups.Add(new Webprofusion.Scalex.Music.ChordDefinition.ctor(0, "Common Guitar Chords", "popular", "popular", null));
    chordGroups.AddRange(chordRenderer.GuitarModel.GetAllChordDefinitions());
    var $it2 = chordGroups.GetEnumerator();
    while ($it2.MoveNext()) {
        var c = $it2.get_Current();
        chordDropdown.append($('<option></option>').val(c.get_ShortName()).html(c.get_Name()));
    }

    //set defaults
    scaleDropdown.val("Minor");
    tuningDropdown.val("6 String Guitar - Standard");
    keyDropdown.val("E");
    chordDropdown.val("popular");
}

function initScales() {
    scaleRenderer.GuitarModel.SetNumberOfFrets(24);

    refreshScaleDiagram();
}

function initChords() {
    refreshChordDiagram();
}

function scrollScaleDiagramLeft() {
    if (!scalePortraitMode) {
        $("#carousel_images").css("left", "0");
    } else {
        $("#carousel_images").css("top", "0");
    }
}

function scrollScaleDiagramRight() {
    if (!scalePortraitMode) {
        $("#carousel_images").css("left", "-450px");
    } else {
        $("#carousel_images").css("top", "-450px");
    }
}

function initApp() {
    Compile();

    currentAppSettings = new Webprofusion.Scalex.CurrentAppSettings.ctor();
    guitarModel = new Webprofusion.Scalex.Guitar.GuitarModel.ctor$$CurrentAppSettings(currentAppSettings);

    scaleRenderer = new Webprofusion.Scalex.Rendering.ScaleDiagramRenderer.ctor$$CurrentAppSettings(currentAppSettings);
    scaleRenderer.GuitarModel = guitarModel; //new Webprofusion.Scalex.Guitar.GuitarModel.ctor$$CurrentAppSettings(currentAppSettings);

    chordRenderer = new Webprofusion.Scalex.Rendering.ChordDiagramRenderer.ctor();
    chordRenderer.GuitarModel = guitarModel;

    //$("#carousel-1").click(scrollScaleDiagramLeft);
    //$("#carousel-2").click(scrollScaleDiagramRight);
    var scaleDiagramElement = document.getElementById('diagram-scrollwrapper');
    var hammertime = Hammer(scaleDiagramElement).on("swiperight", function (event) {
        scrollScaleDiagramLeft();
    });
    hammertime = Hammer(scaleDiagramElement).on("swipeleft", function (event) {
        scrollScaleDiagramRight();
    });

    $("#toggle-portrait").click(function () {
        scalePortraitMode = !scalePortraitMode;
        changeOrientation();
    });

    //setup buttons to change tabs
    $('a[href="#tab-home"]').click(function (e) {
        //e.preventDefault();
        //$('#app-tabs a[href="#tab-home"]').tab('show');
        hideSideNav();
    });

    $('a[href="#tab-chords"]').click(function (e) {
        //$('#app-tabs a[href="#tab-chords"]').tab('show');
        hideSideNav();

        //without this workaround the svg drawn by raphaeljs will have offset text because it's tripped up by the containing tab being initally hidden
        if (!initialised_chords) {
            setTimeout(function () {
                initChords();
            }, 100);
        } else {
            setTimeout(function () {
                refreshChordDiagram();
            }, 100);
        }
    });

    $('a[href="#tab-scales"]').click(function (e) {
        //e.preventDefault();
        //$('#app-tabs a[href="#tab-scales"]').tab('show');
        hideSideNav();

        //without this workaround the svg drawn by raphaeljs will have offset text because it's tripped up by the containing tab being initally hidden
        if (!initialised_scales) {
            setTimeout(function () {
                initScales();
            });
        } else {
            setTimeout(function () {
                refreshScaleDiagram();
            });
        }
    });

    $('a[href="#tab-tuning"]').click(function (e) {
        //e.preventDefault();
        $('#app-tabs a[href="#tab-tuning"]').tab('show');
        hideSideNav();
    });

    $('a[href="#tab-settings"]').click(function (e) {
        // e.preventDefault();
        $('#app-tabs a[href="#tab-settings"]').tab('show');
        hideSideNav();
    });

    $('a[href="#tab-band"]').click(function (e) {
        //var ref = window.open('http://soundshed.com?client=android.app.scalex', '_blank', 'location=yes');
        hideSideNav();
    });

    if (enableTuning) {
        //init audio
        audioController.init();

        $('a[href="#tuning-E2"]').click(function () {
            //audioController.cordovaMediaTest();
            audioController.playNote("E2");
        });

        $('a[href="#tuning-A2"]').click(function () {
            audioController.playNote("A2");
            //audioController.playNote(0, 2, 1000, 1); //A
        });

        $('a[href="#tuning-D3"]').click(function () {
            audioController.playNote("D3");
            //audioController.playNote(4, 3, 1000, 2); //D
        });

        $('a[href="#tuning-G3"]').click(function () {
            audioController.playNote("G3");
            //audioController.playNote(10, 3, 1000, 2); //G
        });

        $('a[href="#tuning-B3"]').click(function () {
            audioController.playNote("B3");
            //audioController.playNote(2, 3, 1000, 2); //B
        });

        $('a[href="#tuning-E4"]').click(function () {
            audioController.playNote("E4");
            //audioController.playNote(7, 4, 1000, 2); //E
        });

        $('a[href="#metronome-120"]').click(function () {
            audioController.metronomeTempo = 120;
            audioController.isMetronomeActive = true;
            audioController.playMetronomeClick();
        });

        $('a[href="#metronome-stop"]').click(function () {
            audioController.isMetronomeActive = false;
        });
    } else {
    }

    initSettings();
}

$(document).ready(function () {
    initApp();
});

// Wait for Cordova to load
//
document.addEventListener("deviceready", onDeviceReady, false);

// Cordova is ready
//
function onDeviceReady() {
    document.addEventListener("backbutton", onBackKeyDown, false);
    isCordova = true;
}

function onBackKeyDown() {
    hideSideNav();
    hideDialogs();
    backButtonCount++;

    if (isCordova) {
        if (backButtonCount > 2 && timeLastBackButton != null) {
            var currTime = new Date();
            if ((currTime - timeLastBackButton) < 1000) {
                //confirm app exit
                if (confirm("Exit Guitar Toolkit?")) {
                    navigator.app.exitApp();
                }
            }
        }
        timeLastBackButton = new Date();
    }
}
//# sourceMappingURL=Scalex.App.js.map
