
Webprofusion.Scalex.Audio = function () {
    "use strict";
    var self = {};
    var context, soundSource, soundBuffer, oscillators;

    self.isWebAudioAvailable = false;
    self.isCordovaMediaAvailable = false;
    self.isSoundManagerAvailable = false;

    var maxVoices = 5;

    self.isMetronomeActive = false;
    self.metronomeTempo = 120;

    function checkSoundManagerAvailable() {
        self.isSoundManagerAvailable = false;
        if (typeof (soundManager) == "undefined") return false;
        else {
            self.isSoundManagerAvailable = true;
            return true;
        }
    }

    function checkCordovaMediaAvailable() {
        self.isCordovaMediaAvailable = false;
        if (typeof (Media) == "undefined") return false;
        else {
            self.isCordovaMediaAvailable = true;
            return true;
        }
    }

    function checkWebAudioAvailable() {
        self.isWebAudioAvailable = true;
        if (typeof AudioContext == "function") {
            context = new AudioContext();
        } else if (typeof webkitAudioContext == "function") {
            context = new webkitAudioContext();
        } else {
            self.isWebAudioAvailable = false;
        }
        return self.isWebAudioAvailable;
    }

    function init() {

        checkCordovaMediaAvailable();
        checkSoundManagerAvailable();
        checkWebAudioAvailable();

        //setup soundmanager library if available
        if (self.isSoundManagerAvailable) {
            soundManager.setup({
                url: 'js/thirdparty/soundmanager/swf/',
                preferFlah: false,
                onready: function () {
                    audioReady = true;
                },
                // optional: ontimeout() callback for handling start-up failure
                ontimeout: function () {
                    audioReady = false;
                }
            });
        }

        if (self.isWebAudioAvailable) {
            //create oscilators for tone generation
            oscillators = [];

            for (var i = 0; i < maxVoices; i++) {
                oscillators[i] = null;
            }
        }
    }

    function soundTest1() {
        if (context == null) alert("No audio context created.");

        oscillators = [], num = 5, ii;

        for (ii = 0; ii < num; ii++) {
            oscillators[ii] = context.createOscillator();
            oscillators[ii].type = 0;
            oscillators[ii].frequency.value = 2000 + ii * 200;
        }

        var connectIt = function (ii) {
            oscillators[ii].connect(context.destination);
            oscillators[ii].noteOn && oscillators[ii].noteOn(0);
            ii++;
            if (ii < num) {
                setTimeout(function () { connectIt(ii); }, 1000);
            } else {
                setTimeout(function () {
                    for (var jj = 0; jj < num; jj++) {
                        oscillators[jj].disconnect();
                    }
                }, 1000);
            }
        }

        connectIt(0);

    }

    function cordovaMediaTest() {
        // Audio player
        //
        var my_media = new Media("media/audio/guitar-clean/E2.mp3", null, null);
        // Play audio
        my_media.play();
        // Update media position every second
        var mediaTimer = setInterval(function () {
            // get media position
            my_media.getCurrentPosition(
                // success callback
                function (position) {
                    if (position > -1) {
                        console.log((position) + " sec");
                    }
                },
                // error callback
                function (e) {
                    console.log("Error getting pos=" + e);
                }
            );
        }, 1000);
    }

    function playTone(note, octave, durationMS, voiceIndex) {

        if (context == null) alert("No audio context created.");
        if (voiceIndex == null) voiceIndex = 0;
        if (durationMS == null) durationMS = 1000;

        //oscillators[0] = context.createOscillator();
        //oscillators[0].type = 0;


        if (oscillators[voiceIndex] != null && oscillators[voiceIndex].playbackState == 2) {
            oscillators[voiceIndex].noteOff && oscillators[voiceIndex].noteOff(context.currentTime); //if note playing, stop it
        }

        //recreate oscillator
        oscillators[voiceIndex] = context.createOscillator();
        oscillators[voiceIndex].type = 0; //sine wave: https://dvcs.w3.org/hg/audio/raw-file/tip/webaudio/specification.html#Oscillator


        var n = new Webprofusion.Scalex.Music.NoteInstance.ctor(note, octave).get_NoteFrequency();
        logEvent("Note Freq:" + n);

        //oscillators[0].frequency.value = 82.41; //open low E (E2)
        //oscillators[1].frequency.value = 110; //open A (A2)
        oscillators[voiceIndex].frequency.value = parseFloat(n); //open D (D3)


        var playTone = function () {
            oscillators[voiceIndex].connect(context.destination); //connect sound generator to output (move to init?)
            oscillators[voiceIndex].noteOn && oscillators[voiceIndex].noteOn(context.currentTime);

            //schedule note stop in 1 second
            setTimeout(function () {
                oscillators[voiceIndex].noteOff(context.currentTime);
                oscillators[voiceIndex].disconnect();
            }, durationMS);
        }
        setTimeout(function () {
            playTone(0);
        }, 500);
        //playTone(2);
        //playTone(0);
        //playTone(0);

    }

    function playNoteMP3(noteName) {
        var audioFile = 'media/audio/guitar-clean/' + noteName + ".mp3";
        if (self.isSoundManagerAvailable) {
            soundManager.stopAll();
            if (audioReady) {
                var mySound = soundManager.createSound({
                    id: 'note_' + noteName,
                    url: audioFile,
                });
                mySound.play();
            }
        } else if (self.isCordovaMediaAvailable) {
            var my_media = new Media(audioFile, null, null);
            my_media.play();
        } else if (self.isWebAudioAvailable) {
        }
    }

    function playNote(noteName) {
        playNoteMP3(noteName);
    }

    function playClick() {
        var audioFile = "media/audio/percussion/metronome_3.mp3";
        if (self.isSoundManagerAvailable) {
            var mySound = soundManager.createSound({
                id: 'metronome_1',
                url: audioFile
            });

            mySound.play();
        } else if (self.isCordovaMediaAvailable) {
            console.log("Click.");
            var my_media = new Media(audioFile, null, null);
            my_media.play();

        } else if (self.isWebAudioAvailable) {
                //play using HTML5 audio
            playNote(1, 2, 500, 0);
        }
    }

    function stopSounds() {
        if (self.isSoundManagerAvailable) {
            soundManager.stopAll();
        }

        if (self.isCordovaMediaAvailable) {
        }
    }

    function playMetronomeClick() {
        if (self.isMetronomeActive) {
            playClick();
            var timeDelay = (1000 * 60) / self.metronomeTempo;
            setTimeout(function () {
                playMetronomeClick();
            }, timeDelay);
        }
    }

    function stopMetronomeClick() {
        self.isMetronomeActive = false;
    }
    //public methods
    self.init = init;
    self.playNote = playNote;
    self.playMetronomeClick = playMetronomeClick;

    return self;
}